#'ProxiesAttractor
#'
#'@author Carmen Alvarez-Castro, \email{carmen.alvarez-castro@cmcc.it}
#'@author Maria M. Chaves-Montero, \email{mdm.chaves-montero@cmcc.it}
#'@author Veronica Torralba, \email{veronica.torralba@cmcc.it}
#'@author Davide Faranda, \email{davide.faranda@lsce.ipsl.fr}
#'
#'@description This function computes two dinamical proxies of the attractor: 
#'The local dimension (d) and the inverse of the persistence (theta). 
#'These two parameters will be used as a condition for the computation of dynamical 
#'scores to measure predictability and to compute bias correction conditioned  by
#'the dynamics with the function DynBiasCorrection  
#'Funtion based on the matlab code (davide.faranda@lsce.ipsl.fr) used in 
#'"Faranda, D., Alvarez-Castro, M.C., Messori, G., Rodriguez, D., and Yiou, P. (2019). 
#' The hammam effect or how a warm ocean enhances large scale atmospheric predictability.
#' Nature Communications, 10(1), 1316. DOI = https://doi.org/10.1038/s41467-019-09305-8 "
#' Davide Faranda, Gabriele Messori and Pascal Yiou. (2017).
#' Dynamical proxies of North Atlantic predictability and extremes. 
#' Scientific Reports, 7-41278, 2017. 
#'
#'@param data data to create the attractor. Must be a matrix with the timesteps in nrow 
#'and the grids in ncol(dat(time,grids)
# 
#'@param quanti list of arbitrary length of secondary grids. Each secondary grid is to
#' be provided as a list of length 2 with longitudes and latitudes 
#' 
#'@param iplot FALSE by default. If TRUE the function returns a simple plot 
#'with three pannels on the top left a plot for local dimension 'd', on the top 
#'right a plot for the inverse of the persistence 'theta', on the bottom a plot
#'for the 'attractor' plotted with the two properties 'd' (x axis) and 'theta'
#'(y axis)
#'
#'@return dim and theta
#'@import s2dverification
#'@import fields
#'
#'@examples
#'# Example 1: computing the attractor using simple data
#'# Creating an example of matrix data(time,grids):
#'tm=2*6*3 # time
#'gm=4*10 # grids
#'m=matrix(rand(1,tm*gm),nrow=tm,ncol=gm) 
#'qm=0.90 # imposing a threshold
#'Attractor=ProxiesAttractor(data=m,quanti=qm,iplot=FALSE)
#'
#'# Example 2: computing the attractor using simple s2dv data
#'expL <- rnorm(1:200)
#'dim(expL) <- c(member=10,lat = 4, lon = 5)
#'obsL <- c(rnorm(1:180),expL[1,,]*1.2)
#'dim(obsL) <- c(time = 10,lat = 4, lon = 5)
#'time_obsL <- paste(rep("01", 10), rep("01", 10), 1994 : 2003, sep = "-")
#'time_expL <- time_obsL[1]
#'lon <- seq(-1,5,1.5)
#'lat <- seq(30,35,1.5)
#'qm=0.98 # too high for this short dataset, it is possible that doesn't 
#'get the requirement, in that case it would be necessary select a lower qm
#'for instance qm=0.60
#'qm=0.60
#'expL <- s2dv_cube(data = expL, lat = lat, lon = lon, 
#'                  Dates = list(start = time_expL, end = time_expL))
#'obsL <- s2dv_cube(data = obsL, lat = lat, lon = lon,
#'                  Dates = list(start = time_obsL, end = time_obsL))
#'
#'attractor<- CST_ProxiesAttractor(data=obsL,quanti=qm)
#'
#'
#'
#'
CST_ProxiesAttractor <- function(data,quanti,ncores=NULL){
  
  if (!inherits(data, 's2dv_cube')) {
    stop("Parameter 'data' must be of the class 's2dv_cube', ",
         "as output by CSTools::CST_Load.")
  }
  if (any(names(dim(data$data)) %in% 'sdate')) {
    if (any(names(dim(data$data)) %in% 'ftime')) {
      data$data <- MergeDims(data$data, merge_dims = c('ftime', 'sdate'),
                             rename_dim = 'time')
    }
  }
  if (!(any(names(dim(data$data)) %in% 'time'))){
    stop('dimension time is mandatory')
  }
  if (any(names(dim(data$data)) %in% 'lat')) {
    if (any(names(dim(data$data)) %in% 'lon')) {
      data$data <- MergeDims(data$data, merge_dims = c('lon', 'lat'),
                             rename_dim = 'grid')
    }
  }
  if(!(any(names(dim(data$data)) %in% 'grid'))){
    stop('dimension grid is mandatory')
  }  
  
  attractor <-Apply(data$data,target_dims=c('time','grid'), fun=ProxiesAttractor,
                    quanti=quanti,iplot=FALSE, ncores=ncores)
  # rename dimensions
  attractor <- lapply(attractor, FUN=function(x,dimname){names(dim(x))[dimname] <- 'time'
  return(x)}, dimname = which(names(dim(attractor[[1]]))=='dim2'))
  
  return(list(dim=attractor$dim,theta=attractor$theta))
}

ProxiesAttractor <- function(data,quanti, iplot=FALSE,ncores=NULL){
  
  logdista <- apply(data,1,function(x,y){
    -log(colMeans((y-x)^2))},
    y=t(data))
  
  #Computation of theta
  Theta <- function(logdista,quanti){
    #Compute the thheshold corresponding to the quantile
    thresh  <- quantile(logdista,quanti,na.rm=TRUE)
    logdista[which(logdista=='Inf')] <-  NaN
    Li <- which(as.vector(logdista) > as.numeric(thresh))
    #Length of each cluster
    Ti <- diff(Li)
    N <-  length(Ti)
    q <-  1-quanti
    Si <- Ti-1
    Nc <- length(which(Si>0))
    N <-  length(Ti)
    theta <- (sum(q*Si)+N+Nc-sqrt(((sum(q*Si)+N+Nc)^2)-8*Nc*sum(q*Si)))/(2*sum(q*Si))
    #Sort the exceedances  
    logdista <- sort(logdista)
    #Find all the Peaks over Thresholds.
    findidx <- which(as.vector(logdista) > as.numeric(thresh))
    if(length(findidx) < 1){
      stop("quanti too high for the length of the dataset")
    }
    logextr <- logdista[findidx[[1]]:(length(logdista)-1)]
    #The inverse of the dimension is just the average of the exceedances
    dim <- 1/mean(as.numeric(logextr)-as.numeric(thresh))
    return(list(dim=dim,theta=theta))
  }
  names(dim(logdista)) <- c('dim1','dim2')
  proxies <- Apply(data=list(logdista=logdista),
                  target_dims=list('dim1'),fun=Theta,quanti=quanti,ncores = ncores)
  if(iplot==TRUE){
    time=c(1:length(proxies$theta))
    layout(matrix(c(1,3,2,3),2,2))
    plot(time,proxies$dim,xlab='time',ylab='d',main='local dimension',type='l')
    plot(time,proxies$theta,xlab='time',ylab='theta',main='theta') 
    plot(proxies$dim,proxies$theta,col='blue',main="Proxies of the Attractor",xlab="local dimension",ylab="theta",lwd=8,'p')
  }
  
  return(list(dim=proxies$dim,theta=proxies$theta))
}
